/* flow analyzer routine */

#include "data.h"

static double dxc,dx;

static double P_pist(void) {
 return PP[1];
}

static double T_pist(void) {
 return TT[1];
}

static double V_pist(void) {
 return VV[1];
}

static double P_max(void) {
 int k;
 double p=PP[1];
 for(k=2;k<=nx;k++) if(PP[k]>p) p=PP[k];
 return p;
}

static double T_max(void) {
 int k;
 double T=TT[1];
 for(k=2;k<=nx;k++) if(TT[k]>T) T=TT[k];
 return T;
}

static double V_max(void) {
 int k;
 double v=VV[1];
 for(k=2;k<=nx;k++) if(VV[k]>v) v=VV[k];
 return v;
}

static double X_pist(void) {return XB[0];}

static void X_shock(double *xs) {
 int j,k,i;
 double tmp,dp,difp,dpm;
 int jm[3],jlim=4;
 difp=nx*P0/L*1.e-8;
 /* fill in Ftmp with |dP/dx| */
 for(j=1;j<nx;j++) Ftmp[j]=fabs(PP[j+1]-PP[j])/(XC[j+1]-XC[j]);
 /* mark up local maxima */
 for(j=0;j<=nx;j++) Jtmp[j]=0;
 for(k=0,j=2;j<nx-1;j++) if(Ftmp[j]>Ftmp[j-1]+difp && Ftmp[j]>Ftmp[j+1]+difp) {
  if(VV[j+1]-VV[j]<0.) {Jtmp[j]=1;k++;}
 }
 /* determine 3 highest, in that order */
 for(i=0;i<3;i++) {
  if(k==0) {
   if(i>0) jm[i]=jm[i-1];
   else jm[i]=0; 
   continue;
  }
  if(i==1) dpm=dp;
  dp=0.;jm[i]=0;
  for(j=2;j<nx;j++) if(Jtmp[j]) {
   if(Ftmp[j]>dp) {jm[i]=j;dp=Ftmp[j];}
  }
  for(j=jm[i]-jlim;j<=jm[i]+jlim;j++) if(j>=0 && j<=nx) Jtmp[j]=0;
  if(jm[i]<2) jm[i]=0;
  else if(i>0 && dp<dpm*0.01) jm[i]=0;
  if(jm[i]==0) k=0;
  else k--;
  if(jm[i]==0 && i>0) jm[i]=jm[i-1];
 }
 for(i=0;i<3;i++) {
  if(jm[i]==0) xs[i]=XB[0];
  else xs[i]=0.5*(XC[jm[i]]+XC[jm[i]+1]);
 }
}

static double X_flame(void) {
 int j,k;
 double tmp,dp,difp;
 dp=0.;k=0;
 for(j=2;j<=nx;j++) {
  difp=fabs(Rh(FUEL)[j]/RH[j]-Rh(FUEL)[j-1]/RH[j-1]);
  if((tmp=difp/(XC[j]-XC[j-1]))>dp && difp/Rh(FUEL)[j]>0.1) {dp=tmp;k=j-1;}
 }
 if(k==0) return XB[k];
 else return 0.5*(XC[k]+XC[k+1]);
}

void AnalyzeFlow(int *nv, double *tv) {
 int i;
 dxc=XC[2]-XC[1];dx=XB[1]-XB[0];
 tv[0]=t*1.e3;
 tv[1]=P_pist()*1.e-5;
 tv[2]=T_pist();
 tv[3]=V_pist();
 tv[4]=P_max()*1.e-5;
 tv[5]=T_max();
 tv[6]=V_max();
 tv[7]=X_pist()*100.;
 tv[8]=X_flame()*100.;
 X_shock(tv+9);
 for(i=0;i<3;i++) tv[9+i]*=100.;
 *nv=12;
}

/* flow post-processing */
void PostProcFlow(double **To, int so) {
 int k,s,ex=0;
 double *t1,*t1o,*t2,*t2o,tmp;
 double eps=(100*L)/nx*TMP_STEPS;
 for(s=1;s<so;s++) {
  /* determine swaps & exchange */
  t1=To[9]+s;t1o=t1-1;
  t2=To[10]+s;t2o=t2-1;
  if(fabs(*t1-*t2o)<eps && fabs(*t2-*t1o)<eps 
	&& fabs(*t1-*t1o)>eps && fabs(*t2-*t2o)>eps) {
   tmp=(*t1);*t1=(*t2);*t2=tmp;++ex;
  }
  t1=To[9]+s;t1o=t1-1;
  t2=To[11]+s;t2o=t2-1;
  if(fabs(*t1-*t2o)<eps && fabs(*t2-*t1o)<eps 
		&& fabs(*t1-*t1o)>eps && fabs(*t2-*t2o)>eps) {
   tmp=(*t1);*t1=(*t2);*t2=tmp;++ex;
  }
  t1=To[10]+s;t1o=t1-1;
  t2=To[11]+s;t2o=t2-1;
  if(fabs(*t1-*t2o)<eps && fabs(*t2-*t1o)<eps 
	&& fabs(*t1-*t1o)>eps && fabs(*t2-*t2o)>eps) {
   tmp=(*t1);*t1=(*t2);*t2=tmp;++ex;
  }
  /* do something else */
 }
 if(ex) printf("ex=%d\n",ex);
}


