/* The file reading thermo-chemical elements data from data base 
   and preparing work elements structure */

#include "data.h"

void PrepareGas(double wox, double gox, double wfu, double gfu,
	double wpr, double gpr, double wnu, double gnu,
	double qr, double nuox, double arrh, double Tac, double Tme, 
	int *ierr, int *warn, char *errmsg) {
 int c;
 double wmin=0.01,gmin=1.01;
 if(wox<=0.) {(*ierr)++;strcat(errmsg,"Error: Oxidizer molar mass is out of range\n");}
 else if(wox<wmin) {(*warn)++;strcat(errmsg,"Warning: Oxidizer molar mass is out of range [0.01, inf) g/mol\n");}
 if(gox<=1.) {(*ierr)++;strcat(errmsg,"Error: Oxidizer capacity ratio is out of range\n");}
 else if(gox<gmin) {(*warn)++;strcat(errmsg,"Warning: Oxidizer capacity ratio is out of range [1.01, inf)\n");}
 if(wfu<=0.) {(*ierr)++;strcat(errmsg,"Error: Fuel molar mass is out of range\n");}
 else if(wfu<wmin) {(*warn)++;strcat(errmsg,"Warning: Fuel molar mass is out of range [0.01, inf) g/mol\n");}
 if(gfu<=1.) {(*ierr)++;strcat(errmsg,"Error: Fuel capacity ratio is out of range\n");}
 else if(gfu<gmin) {(*warn)++;strcat(errmsg,"Warning: Fuel capacity ratio is out of range [1.01, inf)\n");}
 if(wpr<=0.) {(*ierr)++;strcat(errmsg,"Error: Products molar mass is out of range\n");}
 else if(wpr<wmin) {(*warn)++;strcat(errmsg,"Warning: Products molar mass is out of range [0.01, inf) g/mol\n");}
 if(gpr<=1.) {(*ierr)++;strcat(errmsg,"Error: Products capacity ratio is out of range\n");}
 else if(gpr<gmin) {(*warn)++;strcat(errmsg,"Warning: Products capacity ratio is out of range [1.01, inf)\n");}
 if(wnu<=0.) {(*ierr)++;strcat(errmsg,"Error: Neutral molar mass is out of range\n");}
 else if(wnu<wmin) {(*warn)++;strcat(errmsg,"Warning: Neutral molar mass is out of range [0.01, inf) g/mol\n");}
 if(gnu<=1.) {(*ierr)++;strcat(errmsg,"Error: Neutral capacity ratio is out of range\n");}
 else if(gnu<gmin) {(*warn)++;strcat(errmsg,"Warning: Neutral capacity ratio is out of range [1.01, inf)\n");}
 if(arrh<0.) {(*ierr)++;strcat(errmsg,"Error: Pre-exponential factor is out of range\n");}
 if(Tac<1000.) {(*warn)++;strcat(errmsg,"Warning: Activation temperature is out of range [1000, inf)\n");}
 if(Tme<=300.) {(*warn)++;strcat(errmsg,"Warning: Threshold temperature is out of range (300, inf)\n");}
 if(nuox<0.) {(*ierr)++;strcat(errmsg,"Error: Oxidizer stoicheometric factor is out of range\n");}
 else if(nuox<0.01) {(*warn)++;strcat(errmsg,"Warning: Oxidizer stoicheometric factor is out of range [0.01, inf)\n");}
 if(qr<0.) {(*warn)++;strcat(errmsg,"Warning: Reaction heat is out of range [0, inf)\n");}
 if(*ierr) return;
 Wm[OXID]=wox*1.e-3;Gm[OXID]=gox;Wm[FUEL]=wfu*1.e-3;Gm[FUEL]=gfu;
 Wm[PROD]=wpr*1.e-3;Gm[PROD]=gpr;Wm[NEUT]=wnu*1.e-3;Gm[NEUT]=gnu;
 Nust[OXID]=-nuox;Nust[FUEL]=-1.;Nust[NEUT]=0.;
 Nust[PROD]=-(Nust[FUEL]*Wm[FUEL]+Nust[OXID]*Wm[OXID])/Wm[PROD];
 Hm[OXID]=Hm[FUEL]=Hm[NEUT]=0.;
 Hm[PROD]=-qr*1.e3/(Wm[PROD]*Nust[PROD]);
 for(c=0;c<EC;c++) {
  Cvm[c]=Rgas/(Wm[c]*(Gm[c]-1.));Cpm[c]=Cvm[c]*Gm[c];
 }
 /* calculate coeffs Csw */
 for(c=0;c<EC;c++) Csw[c]=Nust[c]*Wm[c]/(Wm[FUEL]*Wm[OXID]);
 /* reaction cof */
 Arrh=arrh;Tact=Tac;Tmed=Tme;
}
 

/* Calculate Cp, Cv, H0 of a molar composition, per mass */
void GetThermo(double *cp,double *cv,double *h0,double *Y) {
 int k;
 double x;
 if(cp) *cp=0.;
 if(cv) *cv=0.;
 if(h0) *h0=0.;
 for(k=0;k<EC;k++) {
  x=Y[k];
  if(x!=0.) {
   if(cp) *cp+=Cpm[k]*x;
   if(cv) *cv+=Cvm[k]*x;
   if(h0) *h0+=Hm[k]*x;
  }
 }
}

/* Given volumetric shares not normalized, get mass shares normalized */
void GetYbyC(double *Yc,double *Cc) {
 int k;
 double cm=0.;
 for(k=0;k<EC;k++) {Yc[k]=Cc[k]*Wm[k];cm+=Yc[k];}
 for(k=0;k<EC;k++) Yc[k]/=cm;
}

/* Given mass shares not normalized, get volume shares normalized */
void GetCbyY(double *Cc,double *Yc) {
 int k;
 double ym=0.;
 for(k=0;k<EC;k++) {Cc[k]=Yc[k]/Wm[k];ym+=Cc[k];}
 for(k=0;k<EC;k++) Cc[k]/=ym;
}

/* Get pressure using state equation */
double Pressure(double T,double rho,double *Yc) {
 int k;
 double sum=0.;
 for(k=0;k<EC;k++) sum+=Yc[k]/Wm[k];
 return Rgas*T*rho*sum;
}

/* Get density using state equation */
double Density(double T,double p,double *Yc) {
 int k;
 double sum=0.;
 for(k=0;k<EC;k++) sum+=Yc[k]/Wm[k];
 return p/(Rgas*T*sum);
}

/* molar mass */
double MolarMass(double *Yc) {
 int k;
 double sum=0.;
 for(k=0;k<EC;k++) sum+=Yc[k]/Wm[k];
 return 1./sum;
}

/* Arrhenius cofs */
double Funarr1(double T) {
 return Arrh*exp(-Tact/T);
}

double Funarr(double T) {
 if(T<Tmed) return 0.;
 else return Funarr1(T);
}

