/* input-output operations */

#include "data.h"

#define DV(a,n) if(!((a)=dvector(n))) mem++
#define IV(a,n) if(!((a)=ivector(n))) mem++

/* allocate memory */
static int Allocmem(int n) {
 int q,mem=0;
 if(n<=0) return 0;
 ++n;
 for(q=0;q<EQ;q++) {DV(Un[q],n);DV(Fc[q],n);DV(Fx[q],n);}
 for(q=0;q<EG;q++) DV(Gr[q],n);
 for(q=0;q<EA;q++) DV(Ua[q],n);
 DV(Ftmp,n);IV(Jtmp,n);
 for(q=0;q<NTout;q++) DV(Tout[q],STout);
 return mem==0;
}

/* process parameters */

#define BUFFSIZ	256
static char buf[BUFFSIZ];
#define RW	if(ReadWord(buf,hnd)==EOF) goto PCLOSE
#define GI(v) 	RW;GetInt(&(v),buf)
#define GD(v) 	RW;GetDbl(&(v),buf)
#define GU(v) 	RW;GetUint(&(v),buf)
#define GS(v)	RW;strcpy((v),buf)	

/* initialize parameters, as it is read. */
void InitParams(void) {
 nx=1000;
 L=200.;Tp=1000.;vp1=350.;vp2=0.;tvp=1.;qw=0.;
 is_Tp=0;
 T0=300.;P0=1.;
 Cfu=1.;Cox=2.;Cpr=0.;Cnu=2.;
 Wfu=16.;Wox=32.;Wpr=18.;Wnu=28.;
 Gox=1.4;Gfu=1.4;Gpr=1.2;Gnu=1.4;
 Qr=250.;Eta=1.e-5;Pran=0.72;Schm=0.7;
 Ar=1.e7;Ta=7000.;Tm=450.;
 Nuox=1.5;
 Ka0=0.01;Le0=1.;
 Sigcour=0.3;
 ntimes=10;timcalc=1.;
 strcpy(out_root,"root");
}

/* reading the parameters set from a file */
int ReadParams(char *file) {
 int ccode=0;
 FILE *hnd=fopen(file,"rt");
 if(!hnd) return 0;
 RW;RW;	/* 2 lines comments for the file */
 GI(nx);GD(L);GD(vp1);GD(vp2);GD(tvp);GI(is_Tp);GD(Tp);GD(qw);RW;
 GD(T0);GD(P0);GD(Cox);GD(Cfu);GD(Cpr);GD(Cnu);RW;
 GD(Wox);GD(Gox);
 GD(Wfu);GD(Gfu);
 GD(Wpr);GD(Gpr);
 GD(Wnu);GD(Gnu);RW;
 GD(Eta);GD(Pran);GD(Schm);RW;
 GD(Qr);GD(Nuox);GD(Ar);GD(Ta);GD(Tm);RW;
 GD(Ka0);GD(Le0);RW;
 GD(Sigcour);GI(ntimes);GD(timcalc);RW;
 GS(out_root);RW;
 GI(ccode);
 fclose(hnd);
 return ccode==123456789;
PCLOSE: fclose(hnd);return EOF;
}

#define WI(var,msg) fprintf(hnd," %d\t" msg "\n",var)
#define WD0(var,msg) fprintf(hnd," %4.0f\t" msg "\n",var)
#define WD1(var,msg) fprintf(hnd," %4.1f\t" msg "\n",var)
#define WD2(var,msg) fprintf(hnd," %4.2f\t" msg "\n",var)
#define WD(var,msg) fprintf(hnd," %e\t" msg "\n",var)
#define WN fprintf(hnd,"\n")

/* write'em, before preparation */
int WriteParams(char *file) {
 FILE *hnd=fopen(file,"wt");
 if(!hnd) return 0;
 fprintf(hnd,"Piston motion ver p0.4\nProblem parameters\n");
 WI(nx,"Cells number");
 WD1(L,"Well initial length, cm");
 WD1(vp1,"Piston velocity (1), m/s");
 WD1(vp2,"Piston velocity (2), m/s");
 WD1(tvp,"Piston velocity change moment, ms");
 WI(is_Tp,"Piston temperature set up: 1 or 0");
 WD1(Tp,"Piston temperature, K");
 WD1(qw,"Heat flux from piston, W/cm2");
 WN;
 WD1(T0,"Initial temperature, K");
 WD1(P0,"Initial pressure, bar");
 WD2(Cox,"Initial oxidizer molar share");
 WD2(Cfu,"Initial fuel molar share");
 WD2(Cpr,"Initial product molar share");
 WD2(Cnu,"Initial neutral molar share");
 WN;
 WD2(Wox,"Oxidizer molar mass, g/mol");
 WD2(Gox,"Oxidizer capacity ratio");
 WD2(Wfu,"Fuel molar mass, g/mol");
 WD2(Gfu,"Fuel capacity ratio");
 WD2(Wpr,"Product molar mass, g/mol");
 WD2(Gpr,"Product capacity ratio");
 WD2(Wnu,"Neutral molar mass, g/mol");
 WD2(Gnu,"Neutral capacity ratio");
 WN;
 WD(Eta,"Dynamical viscosity, Pa*s");
 WD2(Pran,"Prandtl number");
 WD2(Schm,"Schmidt number");
 WN;
 WD1(Qr,"Reaction heat, kJ/mol");
 WD1(Nuox,"Oxidizer stoicheometry coefficient");
 WD(Ar,"Pre-exponential factor");
 WD1(Ta,"Activation temperature, K");
 WD1(Tm,"Threshold temperature, K");
 WN;
 WD2(Ka0,"Initial turbulent energy, J/kg");
 WD1(Le0,"Initial mixing length, cm");
 WN;
 WD2(Sigcour,"CFL criterion factor");
 WI(ntimes,"Number of spatial outputs");
 WD1(timcalc,"Calculation time, ms");
 WN;
 fprintf(hnd," %s\tOutput files root\n",out_root);
 WN;
 fprintf(hnd,"123456789\n");
 fclose(hnd);
 return 1;
}

void PrepareParams(int *ierr, int *warn, char *errmsg) {
 double tz,Lp;
 /* allocate memory */
 if(nx<2 || !Allocmem(nx)) {(*ierr)++;strcat(errmsg,"Error: Memory problems or number of cells is out of range\n");}
 /* check & rescale parameters, except for gas props, with msg */
 if(L<=0.) {(*ierr)++;strcat(errmsg,"Error: Length is out of range\n");}
 if(vp1<-1000. || vp1>2000. || vp2<-1000. || vp2>2000.) {
  (*warn)++;strcat(errmsg,"Warning: Piston velocity is out of range [-1000, 2000] m/s\n");
 }
 if(P0<=0.) {(*ierr)++;strcat(errmsg,"Error: Initial pressure is out of range\n");}
 else if(P0<0.01 || P0>1000.) {
  (*warn)++;strcat(errmsg,"Warning: Initial pressure is out of range [0.01, 1000] bar\n");
 }
 if(T0<=0.) {(*ierr)++;strcat(errmsg,"Error: Initial temperature is out of range\n");}
 else if(T0<100. || T0>1000.) {
  (*warn)++;strcat(errmsg,"Warning: Initial temperature is out of range [100, 1000] K\n");
 }
 if(is_Tp) {
  if(Tp<=0.) {(*ierr)++;strcat(errmsg,"Error: Piston temperature is out of range\n");}
  else if(Tp<10. || Tp>10000.) {
   (*warn)++;strcat(errmsg,"Warning: Piston temperature is out of range [10, 10000] K\n");
  }
 }
 else {
  if(qw<-300.) {(*warn)++;strcat(errmsg,"Warning: Heat flux from piston is out of range\n");}
 }
 if(Cox<0.) {(*ierr)++;strcat(errmsg,"Error: Initial oxidizer share is negative\n");}
 if(Cfu<0.) {(*ierr)++;strcat(errmsg,"Error: Initial fuel share is negative\n");}
 if(Cpr<0.) {(*ierr)++;strcat(errmsg,"Error: Initial products share is negative\n");}
 if(Cnu<0.) {(*ierr)++;strcat(errmsg,"Error: Initial neutral share is negative\n");}
 if(Cox+Cfu+Cpr+Cnu<=0.) {(*ierr)++;strcat(errmsg,"Error: No initial components are set up\n");}
 if(Eta<=0.) {(*ierr)++;strcat(errmsg,"Error: Viscosity is out of range\n");}
 else if(Eta>1.) {(*warn)++;strcat(errmsg,"Warning: Viscosity is out of range (0, 1] Pa*s\n");}
 if(Pran<=0.) {(*ierr)++;strcat(errmsg,"Error: Prandtl number is out of range\n");}
 if(Schm<=0.) {(*ierr)++;strcat(errmsg,"Error: Schmidt number is out of range\n");}
 if(Ka0<=0.) {(*ierr)++;strcat(errmsg,"Error: Initial turbulent energy is out of range\n");}
 else if(Ka0<KAMIN || Ka0>1.e5) {
  (*warn)++;strcat(errmsg,"Warning: Initial turbulent energy is out of range [0.00001, 100000] J/kg\n");
 }
 if(Le0<=0.) {(*ierr)++;strcat(errmsg,"Error: Initial mixing length is out of range\n");}
 if(ntimes<=0) {(*ierr)++;strcat(errmsg,"Error: Outputs number is out of range\n");}
 else if(ntimes>10) {(*warn)++;strcat(errmsg,"Warning: Outputs number is out of range [1, 10]\n");}
 if(timcalc<0.) {(*ierr)++;strcat(errmsg,"Error: Negative calculation time\n");}
 if(Sigcour<=0. || Sigcour>0.5) {
  (*ierr)++;strcat(errmsg,"Error: Courant multiplier is out of range\n");
 }
 else if(Sigcour>0.3) {
  (*warn)++;strcat(errmsg,"Warning: Courant multiplier is out of range (0, 0.3]\n");
 }
 /* change params to SI units */
 L*=0.01;P0*=1.e5;tvp*=1.e-3;timcalc*=1.e-3;
 Le0*=0.01;qw*=1.e4;
 if((tz=tvp)<0.) tz=0.;
 else if(tz>timcalc) tz=timcalc;
 Lp=tz*vp1;
 if(Lp>=L) {(*ierr)++;strcat(errmsg,"Error: Piston intermediate position is out of range\n");}
 else if(Lp<-2.*L || Lp>0.75*L) {
  (*warn)++;strcat(errmsg,"Warning: Piston intermediate position is out of range [-2L;0.75L]\n");
 }
 Lp+=vp2*(timcalc-tz);
 if(Lp>=L) {(*ierr)++;strcat(errmsg,"Error: Piston final position is out of range\n");}
 else if(Lp<-2.*L || Lp>0.75*L) {
  (*warn)++;strcat(errmsg,"Warning: Piston final position is out of range [-2L;0.75L]\n");
 }
 /* prepare species database */
 PrepareGas(Wox,Gox,Wfu,Gfu,Wpr,Gpr,Wnu,Gnu,Qr,Nuox,Ar,Ta,Tm,ierr,warn,errmsg);
 /* finish the message */
 if(*ierr || *warn) {
  sprintf(buf,"Total %d errors, %d warnings\n",*ierr,*warn);
  strcat(errmsg,buf);
 }
}

static void PrintArr(FILE *hnd,char *tit,char *form,int narr,
	double *arr,double mul) {
 int k;
 char *fdefault="%f";
 if(!form) form=fdefault;
 fprintf(hnd,"%s=[",tit);
 for(k=0;k<narr;k++) {
  if(k>0) fprintf(hnd,",");
  fprintf(hnd,form,arr[k]*mul);
 }
 fprintf(hnd,"];\n\n");
}

/* output spatial distribution of parameters */
int OutSpatial(int mode,int vnum,char *file) {
 int k;
 double *var=Fc[0];
 static FILE *hspa;
 switch(mode) {
  case FILE_INIT:
   hspa=fopen(file,"wt");
   return hspa!=NULL;
  case FILE_BODY:
   fprintf(hspa,"t%d=%f;\n\n",vnum,t);
   sprintf(buf,"D%d",vnum);PrintArr(hspa,buf,"%5.2f",nx,XC+1,100.);
   for(k=1;k<=nx;k++) var[k]=PP[k]-2.*RH[k]*KA[k]/3.;
   sprintf(buf,"P%d",vnum);PrintArr(hspa,buf,0,nx,var+1,1.e-5);
   sprintf(buf,"T%d",vnum);PrintArr(hspa,buf,"%7.2f",nx,TT+1,1.);
   sprintf(buf,"V%d",vnum);PrintArr(hspa,buf,0,nx,VV+1,1.);
   for(k=1;k<=nx;k++) var[k]=Rh(FUEL)[k]/RH[k];
   sprintf(buf,"Y%d",vnum);PrintArr(hspa,buf,0,nx,var+1,100.);
   sprintf(buf,"K%d",vnum);PrintArr(hspa,buf,0,nx,KA+1,1.);
   sprintf(buf,"E%d",vnum);PrintArr(hspa,buf,0,nx,EP+1,1.);
   for(k=1;k<=nx;k++) var[k]=Eta/RH[k]+VT[k];
   sprintf(buf,"Vi%d",vnum);PrintArr(hspa,buf,0,nx,var+1,1.);
   return 1;
  case FILE_CLOSE:
   fclose(hspa);
   return 1;
 }
 return 0;
}

/* output temporal distribution of parameters */
int OutTemporal(int mode,char *file) {
 int k,s;
 static FILE *htemp;
 static double TempVars[NTout];
 static int nTempVars,sTempOut,sTempAlloc;
 switch(mode) {
  case FILE_INIT:
   sTempOut=0;sTempAlloc=STout;
   return 1;
  case FILE_BODY:
   AnalyzeFlow(&nTempVars,TempVars);
   if(sTempOut>=sTempAlloc) {
    if((sTempAlloc+=STout)>MAX_TMP_STEPS) return 0;
    for(k=0;k<nTempVars;k++) 
	if(!(Tout[k]=(double *)realloc(Tout[k],sTempAlloc*sizeof(double))))
		return 0;
   }
   for(k=0;k<nTempVars;k++) Tout[k][sTempOut]=TempVars[k];
   sTempOut++;
   return 1;
  case FILE_CLOSE:
   /* post - processing */
   PostProcFlow(Tout,sTempOut);
   htemp=fopen(file,"wt");
   if(!htemp) return 0;
   fprintf(htemp,"Temp=[\n");
   for(s=0;s<sTempOut;s++) {
    for(k=0;k<nTempVars;k++) fprintf(htemp," %f",Tout[k][s]);
    fprintf(htemp,";\n");
   }
   fprintf(htemp,"];\n\n");
   fclose(htemp);
   return 1;
 }
 return 0;
}

/* onto screen */
int OutScreen(void) {
 fprintf(stderr,"s=%d t=%10.7e dt=%9.3e p=%7.4f\n",
	step,t,dt,PP[1]*1.e-5);
 return 1;
}

/* into the log file */
int OutLog(int mode, char *file) {
 static time_t tstart,tfinish;
 static struct tm tmbuf;
 static FILE *hlog;
 switch(mode) {
  case FILE_INIT:
   tstart=time(NULL);
   return 1;
  case FILE_BODY:
  case FILE_CLOSE:
   tfinish=time(NULL);
   hlog=fopen(file,"at");
   if(!hlog) return 0;
   _localtime(&tstart,&tmbuf);
   fprintf(hlog,"Output root: %s\n",out_root);
   fprintf(hlog,"Started: %sElapsed time %d seconds\n\n",
	asctime(&tmbuf),(int)difftime(tfinish,tstart));
   fclose(hlog);
   return 1;
 }
 return 0;
}

/* into the err msg file */
int OutErr(int mode, char *data) {
 static FILE *herr;
 switch(mode) {
  case FILE_INIT: herr=fopen(data,"wt");
   return (herr)?1:0;
  case FILE_BODY: fprintf(herr,data);
   return 1;
  case FILE_CLOSE: fclose(herr);
   return 1;
 }
 return 0;
}


