/* input-output inclusion */

/* species */
#include "gases.h"
/* EC = number of species defined in gases.h */

/* the primitive variables */
enum{DENS,VELO=EC,TEMP,KAHA,EPSI,EQ};
/* EQ = number of balance equations */

/* the grid */
enum{XGRD,VGRD,EG};
/* EG = number of grid arrays */
 
/* arrays */
Global double *Un[EQ],*Gr[EG];

/* abbreviations */
#define Rh(c) 	Un[DENS+(c)]
#define VV	Un[VELO]
#define TT	Un[TEMP]
#define KA	Un[KAHA]
#define EP	Un[EPSI]
#define XB	Gr[XGRD]
#define UB	Gr[VGRD]

/* derived variables & special variables */
enum{GDENS,PRES,SOUND,XCELL,VIST,ENER,ENTHA,EA};

Global double *Ua[EA];

#define RH	Ua[GDENS]
#define PP	Ua[PRES]
#define AA	Ua[SOUND]
#define XC	Ua[XCELL]
#define VT	Ua[VIST]
#define EE	Ua[ENER]

/* The fluxes & other additional vars */
Global double *Fc[EQ],*Fx[EQ],*Ftmp;
Global int *Jtmp;

/* temporary output array */
#define NTout	13
#define STout	500
Global double *Tout[NTout];

/* variables */
Global double t,dt;
Global int step;

/* problem parameters */

/* grid parameters */
Global int nx;
Global double L,Tp,vp1,vp2,tvp,qw;
Global int is_Tp;
/* initial state */
Global double T0,P0,Cox,Cfu,Cpr,Cnu;
/* thermodynamics, transport */
Global double Wox,Wfu,Wpr,Wnu;
Global double Gox,Gfu,Gpr,Gnu;
Global double Qr,Eta,Pran,Schm;
/* Chemical kinetics */
Global double Ar,Ta,Tm;
Global double Nuox;
/* initial turb pars */
Global double Ka0,Le0;
/* Runtime */
Global double Sigcour,timcalc;
Global int ntimes;
/* output file root */
Global char out_root[128];

/* initialize parameters */
extern void InitParams(void);
/* reading the parameters set from a file */
extern int ReadParams(char *file);
extern int WriteParams(char *file);
/* preparing them */
extern void PrepareParams(int *ierr, int *warn, char *errmsg);

/* output temporal data */
enum{FILE_INIT,FILE_BODY,FILE_CLOSE};
extern int OutTemporal(int mode,char *file);
/* output distribution */
extern int OutSpatial(int mode,int vnum,char *file);
/* output onto screen */
extern int OutScreen(void);
/* log file, err file */
extern int OutLog(int mode,char *file);
extern int OutErr(int mode,char *data);

